package sun.awt.shell;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.Callable;
import sun.misc.ThreadGroupUtils;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import sun.awt.windows.WToolkit;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.stream.Stream;
import java.util.Arrays;
import sun.awt.OSInfo;
import java.util.ArrayList;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.List;
import java.io.File;
import java.awt.Image;
import sun.util.logging.PlatformLogger;

public class Win32ShellFolderManager2 extends ShellFolderManager
{
    private static final PlatformLogger log;
    private static final int VIEW_LIST = 2;
    private static final int VIEW_DETAILS = 3;
    private static final int VIEW_PARENTFOLDER = 8;
    private static final int VIEW_NEWFOLDER = 11;
    private static final Image[] STANDARD_VIEW_BUTTONS;
    private static Win32ShellFolder2 desktop;
    private static Win32ShellFolder2 drives;
    private static Win32ShellFolder2 recent;
    private static Win32ShellFolder2 network;
    private static Win32ShellFolder2 personal;
    private static File[] roots;
    private static List topFolderList;
    
    public Win32ShellFolderManager2() {
    }
    
    @Override
    public ShellFolder createShellFolder(File file) throws FileNotFoundException {
        try {
            return createShellFolder(getDesktop(), file);
        }
        catch (InterruptedException ex) {
            throw new FileNotFoundException("Execution was interrupted");
        }
    }
    
    static Win32ShellFolder2 createShellFolder(Win32ShellFolder2 win32ShellFolder2, File file) throws FileNotFoundException, InterruptedException {
        long displayName;
        try {
            displayName = win32ShellFolder2.parseDisplayName(file.getCanonicalPath());
        }
        catch (IOException ex) {
            displayName = 0L;
        }
        if (displayName == 0L) {
            throw new FileNotFoundException("File " + file.getAbsolutePath() + " not found");
        }
        try {
            return createShellFolderFromRelativePIDL(win32ShellFolder2, displayName);
        }
        finally {
            Win32ShellFolder2.releasePIDL(displayName);
        }
    }
    
    static Win32ShellFolder2 createShellFolderFromRelativePIDL(Win32ShellFolder2 win32ShellFolder2, long nextPIDLEntry) throws InterruptedException {
        while (nextPIDLEntry != 0L) {
            long copyFirstPIDLEntry = Win32ShellFolder2.copyFirstPIDLEntry(nextPIDLEntry);
            if (copyFirstPIDLEntry != 0L) {
                win32ShellFolder2 = new Win32ShellFolder2(win32ShellFolder2, copyFirstPIDLEntry);
                nextPIDLEntry = Win32ShellFolder2.getNextPIDLEntry(nextPIDLEntry);
            }
            else {
                break;
            }
        }
        return win32ShellFolder2;
    }
    
    private static Image getStandardViewButton(int n) {
        Image image = Win32ShellFolderManager2.STANDARD_VIEW_BUTTONS[n];
        if (image != null) {
            return image;
        }
        BufferedImage bufferedImage = new BufferedImage(16, 16, 2);
        bufferedImage.setRGB(0, 0, 16, 16, Win32ShellFolder2.getStandardViewButton0(n), 0, 16);
        return Win32ShellFolderManager2.STANDARD_VIEW_BUTTONS[n] = bufferedImage;
    }
    
    static Win32ShellFolder2 getDesktop() {
        if (Win32ShellFolderManager2.desktop == null) {
            try {
                Win32ShellFolderManager2.desktop = new Win32ShellFolder2(0);
            }
            catch (SecurityException ex) {}
            catch (IOException | InterruptedException t) {
                if (Win32ShellFolderManager2.log.isLoggable(PlatformLogger.Level.WARNING)) {
                    Win32ShellFolderManager2.log.warning("Cannot access 'Desktop'", t);
                }
            }
        }
        return Win32ShellFolderManager2.desktop;
    }
    
    static Win32ShellFolder2 getDrives() {
        if (Win32ShellFolderManager2.drives == null) {
            try {
                Win32ShellFolderManager2.drives = new Win32ShellFolder2(17);
            }
            catch (SecurityException ex) {}
            catch (IOException | InterruptedException t) {
                if (Win32ShellFolderManager2.log.isLoggable(PlatformLogger.Level.WARNING)) {
                    Win32ShellFolderManager2.log.warning("Cannot access 'Drives'", t);
                }
            }
        }
        return Win32ShellFolderManager2.drives;
    }
    
    static Win32ShellFolder2 getRecent() {
        if (Win32ShellFolderManager2.recent == null) {
            try {
                String fileSystemPath = Win32ShellFolder2.getFileSystemPath(8);
                if (fileSystemPath != null) {
                    Win32ShellFolderManager2.recent = createShellFolder(getDesktop(), new File(fileSystemPath));
                }
            }
            catch (SecurityException ex) {}
            catch (InterruptedException | IOException t) {
                if (Win32ShellFolderManager2.log.isLoggable(PlatformLogger.Level.WARNING)) {
                    Win32ShellFolderManager2.log.warning("Cannot access 'Recent'", t);
                }
            }
        }
        return Win32ShellFolderManager2.recent;
    }
    
    static Win32ShellFolder2 getNetwork() {
        if (Win32ShellFolderManager2.network == null) {
            try {
                Win32ShellFolderManager2.network = new Win32ShellFolder2(18);
            }
            catch (SecurityException ex) {}
            catch (IOException | InterruptedException t) {
                if (Win32ShellFolderManager2.log.isLoggable(PlatformLogger.Level.WARNING)) {
                    Win32ShellFolderManager2.log.warning("Cannot access 'Network'", t);
                }
            }
        }
        return Win32ShellFolderManager2.network;
    }
    
    static Win32ShellFolder2 getPersonal() {
        if (Win32ShellFolderManager2.personal == null) {
            try {
                String fileSystemPath = Win32ShellFolder2.getFileSystemPath(5);
                if (fileSystemPath != null) {
                    Win32ShellFolderManager2.personal = getDesktop().getChildByPath(fileSystemPath);
                    if (Win32ShellFolderManager2.personal == null) {
                        Win32ShellFolderManager2.personal = createShellFolder(getDesktop(), new File(fileSystemPath));
                    }
                    if (Win32ShellFolderManager2.personal != null) {
                        Win32ShellFolderManager2.personal.setIsPersonal();
                    }
                }
            }
            catch (SecurityException ex) {}
            catch (InterruptedException | IOException t) {
                if (Win32ShellFolderManager2.log.isLoggable(PlatformLogger.Level.WARNING)) {
                    Win32ShellFolderManager2.log.warning("Cannot access 'Personal'", t);
                }
            }
        }
        return Win32ShellFolderManager2.personal;
    }
    
    @Override
    public Object get(String s) {
        if (s.equals("fileChooserDefaultFolder")) {
            Win32ShellFolder2 win32ShellFolder2 = getPersonal();
            if (win32ShellFolder2 == null) {
                win32ShellFolder2 = getDesktop();
            }
            return checkFile(win32ShellFolder2);
        }
        else if (s.equals("roots")) {
            if (Win32ShellFolderManager2.roots == null) {
                Win32ShellFolder2 desktop2 = getDesktop();
                if (desktop2 != null) {
                    Win32ShellFolderManager2.roots = new File[] { desktop2 };
                }
                else {
                    Win32ShellFolderManager2.roots = (File[])(File[])super.get(s);
                }
            }
            return checkFiles(Win32ShellFolderManager2.roots);
        }
        else if (s.equals("fileChooserComboBoxFolders")) {
            Win32ShellFolder2 desktop = getDesktop();
            if (desktop != null && checkFile(desktop) != null) {
                ArrayList list2 = new ArrayList();
                Win32ShellFolder2 drives = getDrives();
                Win32ShellFolder2 recent = getRecent();
                if (recent != null && OSInfo.getWindowsVersion().compareTo(OSInfo.WINDOWS_2000) >= 0) {
                    list2.add(recent);
                }
                list2.add(desktop);
                File[] checkFiles = checkFiles(desktop.listFiles());
                Arrays.sort(checkFiles);
                for (int length2 = checkFiles.length, j = 0; j < length2; ++j) {
                    Win32ShellFolder2 e2 = (Win32ShellFolder2)checkFiles[j];
                    if (!e2.isFileSystem() || (e2.isDirectory() && !e2.isLink())) {
                        list2.add(e2);
                        if (e2.equals(drives)) {
                            File[] checkFiles2 = checkFiles(e2.listFiles());
                            if (checkFiles2 != null && checkFiles2.length > 0) {
                                List<File> list3 = Arrays.asList(checkFiles2);
                                e2.sortChildren(list3);
                                list2.addAll(list3);
                            }
                        }
                    }
                }
                return checkFiles(list2);
            }
            else {
                return super.get(s);
            }
        }
        else if (s.equals("fileChooserShortcutPanelFolders")) {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            ArrayList list = new ArrayList();
            int n2 = 0;
            Object desktopProperty;
            do {
                desktopProperty = defaultToolkit.getDesktopProperty("win.comdlg.placesBarPlace" + n2++);
                try {
                    if ((desktopProperty instanceof Integer)) {
                        list.add(new Win32ShellFolder2((int)desktopProperty));
                    }
                    else if ((desktopProperty instanceof String)) {
                        list.add(this.createShellFolder(new File((String)desktopProperty)));
                    }
                    else {
                        continue;
                    }
                }
                catch (IOException t) {
                    if (Win32ShellFolderManager2.log.isLoggable(PlatformLogger.Level.WARNING)) {
                        Win32ShellFolderManager2.log.warning("Cannot read value = " + desktopProperty, t);
                    }
                    else {
                        continue;
                    }
                }
                catch (InterruptedException t2) {
                    if (Win32ShellFolderManager2.log.isLoggable(PlatformLogger.Level.WARNING)) {
                        Win32ShellFolderManager2.log.warning("Cannot read value = " + desktopProperty, t2);
                    }
                    return new File[0];
                }
            } while (desktopProperty != null);
            if (list.size() == 0) {
                for (File e : new File[] { getRecent(), getDesktop(), getPersonal(), getDrives(), getNetwork() }) {
                    if (e != null) {
                        list.add(e);
                    }
                }
            }
            return checkFiles(list);
        }
        else if (s.startsWith("fileChooserIcon ")) {
            String substring2 = s.substring(s.indexOf(" ") + 1);
            int n;
            if (substring2.equals("ListView") || substring2.equals("ViewMenu")) {
                n = 2;
            }
            else if (substring2.equals("DetailsView")) {
                n = 3;
            }
            else if (substring2.equals("UpFolder")) {
                n = 8;
            }
            else if (substring2.equals("NewFolder")) {
                n = 11;
            }
            else {
                return null;
            }
            return getStandardViewButton(n);
        }
        else if (s.startsWith("optionPaneIcon ")) {
            Win32ShellFolder2.SystemIcon systemIcon;
            if (s == "optionPaneIcon Error") {
                systemIcon = Win32ShellFolder2.SystemIcon.IDI_ERROR;
            }
            else if (s == "optionPaneIcon Information") {
                systemIcon = Win32ShellFolder2.SystemIcon.IDI_INFORMATION;
            }
            else if (s == "optionPaneIcon Question") {
                systemIcon = Win32ShellFolder2.SystemIcon.IDI_QUESTION;
            }
            else if (s == "optionPaneIcon Warning") {
                systemIcon = Win32ShellFolder2.SystemIcon.IDI_EXCLAMATION;
            }
            else {
                return null;
            }
            return Win32ShellFolder2.getSystemIcon(systemIcon);
        }
        else {
            if (s.startsWith("shell32Icon ") || s.startsWith("shell32LargeIcon ")) {
                String substring = s.substring(s.indexOf(" ") + 1);
                try {
                    int int1 = Integer.parseInt(substring);
                    if (int1 >= 0) {
                        return Win32ShellFolder2.getShell32Icon(int1, s.startsWith("shell32LargeIcon "));
                    }
                }
                catch (NumberFormatException ex) {}
            }
            return null;
        }
    }
    
    private static File checkFile(File file) {
        SecurityManager securityManager = System.getSecurityManager();
        return (securityManager == null || file == null) ? file : checkFile(file, securityManager);
    }
    
    private static File checkFile(File file, SecurityManager securityManager) {
        try {
            securityManager.checkRead(file.getPath());
            if ((file instanceof Win32ShellFolder2)) {
                Win32ShellFolder2 win32ShellFolder2 = (Win32ShellFolder2)file;
                if (win32ShellFolder2.isLink()) {
                    Win32ShellFolder2 win32ShellFolder3 = (Win32ShellFolder2)win32ShellFolder2.getLinkLocation();
                    if (win32ShellFolder3 != null) {
                        securityManager.checkRead(win32ShellFolder3.getPath());
                    }
                }
            }
            return file;
        }
        catch (SecurityException ex) {
            return null;
        }
    }
    
    static File[] checkFiles(File[] array) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null || array == null || array.length == 0) {
            return array;
        }
        return checkFiles(Arrays.stream(array), securityManager);
    }
    
    private static File[] checkFiles(List<File> list) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null || list.isEmpty()) {
            return (File[])list.toArray(new File[list.size()]);
        }
        return checkFiles(list.stream(), securityManager);
    }
    
    private static File[] checkFiles(Stream<File> stream, SecurityManager securityManager) {
        return (File[])stream.filter(file/* java.io.File, */ -> checkFile(file, p1) != null).toArray(File[]::new);
    }
    
    @Override
    public boolean isComputerNode(final File file) {
        if (file != null && file == getDrives()) {
            return true;
        }
        String s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return file.getAbsolutePath();
            }
        });
        return s.startsWith("\\\\") && s.indexOf("\\", 2) < 0;
    }
    
    @Override
    public boolean isFileSystemRoot(File file) {
        if (file != null) {
            Win32ShellFolder2 drives = getDrives();
            if ((file instanceof Win32ShellFolder2)) {
                Win32ShellFolder2 win32ShellFolder2 = (Win32ShellFolder2)file;
                if (win32ShellFolder2.isFileSystem()) {
                    if (win32ShellFolder2.parent != null) {
                        return win32ShellFolder2.parent.equals(drives);
                    }
                }
                else {
                    return false;
                }
            }
            String path = file.getPath();
            if (path.length() != 3 || path.charAt(1) != ':') {
                return false;
            }
            File[] listFiles = drives.listFiles();
            return listFiles != null && Arrays.asList(listFiles).contains(file);
        }
        else {
            return false;
        }
    }
    
    static int compareShellFolders(Win32ShellFolder2 win32ShellFolder2, Win32ShellFolder2 win32ShellFolder3) {
        int special = win32ShellFolder2.isSpecial();
        int special2 = win32ShellFolder3.isSpecial();
        if (special || special2) {
            if (Win32ShellFolderManager2.topFolderList == null) {
                ArrayList<Win32ShellFolder2> topFolderList = new ArrayList();
                topFolderList.add(getPersonal());
                topFolderList.add(getDesktop());
                topFolderList.add(getDrives());
                topFolderList.add(getNetwork());
                Win32ShellFolderManager2.topFolderList = topFolderList;
            }
            int index = Win32ShellFolderManager2.topFolderList.indexOf(win32ShellFolder2);
            int index2 = Win32ShellFolderManager2.topFolderList.indexOf(win32ShellFolder3);
            if (index >= 0 && index2 >= 0) {
                return index - index2;
            }
            if (index >= 0) {
                return -1;
            }
            if (index2 >= 0) {
                return 1;
            }
        }
        if (special && special2 == 0) {
            return -1;
        }
        if (special2 && special == 0) {
            return 1;
        }
        return compareNames(win32ShellFolder2.getAbsolutePath(), win32ShellFolder3.getAbsolutePath());
    }
    
    static int compareNames(String s, String s2) {
        int compareToIgnoreCase = s.compareToIgnoreCase(s2);
        if (compareToIgnoreCase != 0) {
            return compareToIgnoreCase;
        }
        return s.compareTo(s2);
    }
    
    @Override
    protected ShellFolder.Invoker createInvoker() {
        return new Win32ShellFolderManager2.ComInvoker();
    }
    
    static native void initializeCom();
    
    static native void uninitializeCom();
    
    static {
        log = PlatformLogger.getLogger("sun.awt.shell.Win32ShellFolderManager2");
        WToolkit.loadLibraries();
        STANDARD_VIEW_BUTTONS = new Image[12];
        Win32ShellFolderManager2.topFolderList = null;
    }
    
    private static class ComInvoker extends ThreadPoolExecutor implements ThreadFactory, ShellFolder.Invoker
    {
        private static Thread comThread;
        
        private ComInvoker() {
            super(1, 1, 0L, TimeUnit.DAYS, new LinkedBlockingQueue());
            this.allowCoreThreadTimeOut(false);
            this.setThreadFactory(this);
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                
                @Override
                public Void run() {
                    Runtime.getRuntime().addShutdownHook(new Thread(shutdownHook));
                    return null;
                }
            });
        }
        
        @Override
        public synchronized Thread newThread(Runnable runnable) {
            return Win32ShellFolderManager2.ComInvoker.comThread = (Thread)AccessController.doPrivileged(()/*  */ -> {
                Win32ShellFolderManager2.ComInvoker.3 o = new Runnable() {
                    @Override
                    public void run() {
                        try {
                            Win32ShellFolderManager2.initializeCom();
                            runnable2.run();
                        }
                        finally {
                            Win32ShellFolderManager2.uninitializeCom();
                        }
                    }
                };
                Thread thread = new Thread(ThreadGroupUtils.getRootThreadGroup(), task, "Swing-Shell");
                thread.setDaemon(true);
                return thread;
            });
        }
        
        @Override
        public <T> T invoke(Callable<T> task) throws Exception {
            if (Thread.currentThread() == Win32ShellFolderManager2.ComInvoker.comThread) {
                return task.call();
            }
            Future<T> submit;
            try {
                submit = this.submit(task);
            }
            catch (RejectedExecutionException ex) {
                throw new InterruptedException(ex.getMessage());
            }
            try {
                return submit.get();
            }
            catch (InterruptedException ex2) {
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        submit.cancel(true);
                        return null;
                    }
                });
                throw ex2;
            }
            catch (ExecutionException ex3) {
                Throwable cause = ex3.getCause();
                if ((cause instanceof Exception)) {
                    throw (Exception)cause;
                }
                if ((cause instanceof Error)) {
                    throw (Error)cause;
                }
                throw new RuntimeException("Unexpected error", cause);
            }
        }
    }
}
